# -*- coding: utf-8 -*-
from c3s_sm.interface import C3S_Nc_Img_Stack
from datetime import datetime
import os
import numpy.testing as nptest
from pygeobase.object_base import  Image


def test_c3s_timestamp_for_daterange():
    parameters = ['sm', 'sm_noise']

    path = os.path.join(os.path.dirname(__file__),
                        'c3s_sm-test-data', 'img', 'TCDR', '060_dailyImages', 'combined')

    ds = C3S_Nc_Img_Stack(path, parameters)


    tstamps = ds.tstamps_for_daterange(datetime(2000, 1, 1),
                                       datetime(2000, 1, 5))
    assert len(tstamps) == 5
    assert tstamps == [datetime(2000, 1, 1),
                       datetime(2000, 1, 2),
                       datetime(2000, 1, 3),
                       datetime(2000, 1, 4),
                       datetime(2000, 1, 5)]

def test_c3s_img_stack_single_img_reading():
    parameters = ['sm']

    path = os.path.join(os.path.dirname(__file__),
                        'c3s_sm-test-data', 'img', 'TCDR', '060_dailyImages', 'combined')

    ds = C3S_Nc_Img_Stack(path, parameters)

    img = ds.read(datetime(2014,1,1)) # type: Image

    nptest.assert_almost_equal(img.data['sm'][167, 785], 0.34659, 4)

def test_c3s_img_stack_multiple_img_reading_TCDR():
    startdate, enddate = datetime(2016,4,1), datetime(2016,6,1)

    parameters = ['sm']

    path = os.path.join(os.path.dirname(__file__),
                        'c3s_sm-test-data', 'img', 'TCDR', '061_monthlyImages', 'combined')

    ds = C3S_Nc_Img_Stack(path, parameters)

    images = ds.iter_images(startdate, enddate)

    for i, img in enumerate(images):
        if i == 0:
            nptest.assert_almost_equal(img.data['sm'][167, 785], 0.32004, 4)
        if i == 1:
            nptest.assert_almost_equal(img.data['sm'][167, 785], 0.31229, 4)
        if i == 2:
            nptest.assert_almost_equal(img.data['sm'][167, 785], 0.31059, 4)


def test_c3s_img_stack_multiple_img_reading_ICDR():
    startdate, enddate = datetime(2017,7,1), datetime(2017,12,1)

    parameters = ['sm']

    path = os.path.join(os.path.dirname(__file__),
                        'c3s_sm-test-data', 'img', 'ICDR', '061_monthlyImages', 'passive')

    ds = C3S_Nc_Img_Stack(path, parameters)

    images = ds.iter_images(startdate, enddate)

    for i, img in enumerate(images):
        if i == 0:
            nptest.assert_almost_equal(img.data['sm'][167, 785], 0.23400, 4)
        if i == 1:
            nptest.assert_almost_equal(img.data['sm'][167, 785], 0.22680, 4)
        if i == 2:
            nptest.assert_almost_equal(img.data['sm'][167, 785], 0.29522, 4)



if __name__ == '__main__':
    test_c3s_img_stack_multiple_img_reading_TCDR()
    test_c3s_img_stack_single_img_reading()
    test_c3s_img_stack_multiple_img_reading_ICDR()
    test_c3s_timestamp_for_daterange()




# -*- coding: utf-8 -*-
"""
Module for a command line interface to convert the C3S data into a
time series format using the repurpose package
"""

import os
import sys
import argparse
from datetime import datetime

from repurpose.img2ts import Img2Ts
from c3s_sm.interface import C3S_Nc_Img_Stack, c3s_filename_template
from c3s_sm.grid import C3SLandGrid, C3SCellGrid
import c3s_sm.metadata as metadata
from c3s_sm.metadata import C3S_daily_tsatt_nc, C3S_dekmon_tsatt_nc
from pygeogrids.grids import BasicGrid

import numpy as np

from parse import parse

from netCDF4 import Dataset


def mkdate(datestring):
    """
    Create date string.

    Parameters
    ----------
    datestring : str
        Date string.
    Returns
    -------
    datestr : datetime
        Date string as datetime.
    """
    if len(datestring) == 10:
        return datetime.strptime(datestring, '%Y-%m-%d')
    if len(datestring) == 16:
        return datetime.strptime(datestring, '%Y-%m-%dT%H:%M')

def str2bool(val):
    if val in ['True', 'true', 't', 'T', '1']:
        return True
    else:
        return False


def parse_filename(data_dir):
    '''
    Take the first file in the passed directory and use its file name to
    retrieve the product type, version number and variables in the file.

    Parameters
    ----------
    inroot : str
        Input root directory

    Returns
    -------
    file_args : dict
        Parsed arguments from file name
    file_vars : list
        Names of parameters in the first detected file
    '''
    template = c3s_filename_template()

    for curr, subdirs, files in os.walk(data_dir):
        for f in files:
            file_args = parse(template, f)
            if file_args is None:
                continue
            else:
                file_args = file_args.named
                file_args['datetime'] = '{datetime}'
                file_vars = Dataset(os.path.join(curr,f)).variables.keys()
                return file_args, list(file_vars)

    raise IOError('No file name in passed directory fits to template')


def reshuffle(input_root, outputpath, startdate, enddate,
              parameters, land_points=True,
              imgbuffer=50):
    """
    Reshuffle method applied to C3S data.
    Parameters
    ----------
    input_root: string
        input path where c3s images were downloaded.
    outputpath : string
        Output path.
    startdate : datetime
        Start date.
    enddate : datetime
        End date.
    parameters: list
        parameters to read and convert
    land_points : bool, optional (default: True)
        Use the land grid to calculate time series on.
        Leads to faster processing and smaller files.
    imgbuffer: int, optional (default: 50)
        How many images to read at once before writing time series.
    """

    if land_points:
        grid = C3SLandGrid()
    else:
        grid = C3SCellGrid()

    gpis, lons, lats, cells = grid.get_grid_points()
    grid_vars = {'gpis': gpis, 'lons':lons, 'lats':lats}
    # repurpose cannot handle masked arrays
    for k, v in grid_vars.items(): # type v: np.ma.MaskedArray
        if isinstance(v, np.ma.MaskedArray):
            grid_vars[k] = v.filled()

    grid = BasicGrid(lon=grid_vars['lons'], lat=grid_vars['lats'], gpis=grid_vars['gpis']).to_cell_grid(5.)

    if parameters is None:
        file_args, file_vars = parse_filename(input_root)
        parameters = [p for p in file_vars if p not in ['lat', 'lon', 'time']]

    input_dataset = C3S_Nc_Img_Stack(data_path=input_root, parameters=parameters,
                                     subgrid=grid, array_1D=True)

    prod_args = input_dataset.fname_args


    kwargs = {'product_sensor_type' : prod_args['sensor_type'].lower(),
              'sub_version' : '.' + prod_args['sub_version'],
              'product_sub_type': prod_args['sub_prod']}

    class_str = "C3S_SM_TS_Attrs_%s" % (prod_args['version'])
    subattr = getattr(metadata, class_str)

    if prod_args['temp_res'] == 'DAILY':
        attrs = C3S_daily_tsatt_nc(subattr, **kwargs)
    else:
        attrs = C3S_dekmon_tsatt_nc(subattr, **kwargs)

    ts_attributes = {}
    global_attributes = attrs.global_attr

    # todo: attrs for all vars or only for the ones that TS were created for.
    for var in parameters:
        ts_attributes.update(attrs.ts_attributes[var])


    if not os.path.exists(outputpath):
        os.makedirs(outputpath)


    reshuffler = Img2Ts(input_dataset=input_dataset, outputpath=outputpath,
                        startdate=startdate, enddate=enddate, input_grid=grid,
                        imgbuffer=imgbuffer, cellsize_lat=5.0,
                        cellsize_lon=5.0, global_attr=global_attributes, zlib=True,
                        unlim_chunksize=1000, ts_attributes=ts_attributes)
    reshuffler.calc()


def parse_args(args):
    """
    Parse command line parameters for C3S reshuffling.

    Parameters
    ----------
    args : list of str
        Command line parameters as list of strings.

    Returns
    -------
    args : argparse.Namespace
        Command line arguments.
    """

    parser = argparse.ArgumentParser(
        description="Convert C3s image data to time series format.")
    parser.add_argument("dataset_root",
                        help='Root of local filesystem where the '
                             'data is stored.')

    parser.add_argument("timeseries_root",
                        help='Root of local filesystem where the timeseries '
                             'should be stored.')

    parser.add_argument("start", type=mkdate,
                        help=("Startdate. In format YYYY-MM-DD"))

    parser.add_argument("end", type=mkdate,
                        help=("Enddate. In format YYYY-MM-DD"))

    parser.add_argument("--parameters", metavar="parameters", default=None,
                        nargs="+",
                        help=("Parameters to reshuffle into time series format. "
                              "E.g. sm for creating soil moisture time series."
                              "If None are passed, all variables from the first image file in the path are used."))

    parser.add_argument("--land_points", type=str2bool, default='False',
                        help=("Set True to convert only land points as defined"
                              " in the C3s land mask (faster and less/smaller files)"))

    parser.add_argument("--imgbuffer", type=int, default=50,
                        help=("How many images to read at once. Bigger "
                              "numbers make the conversion faster but "
                              "consume more memory."))

    args = parser.parse_args(args)
    # set defaults that can not be handled by argparse

    print("Converting data from {} to"
          " {} into folder {}.".format(args.start.isoformat(),
                                      args.end.isoformat(),
                                      args.timeseries_root))

    return args


def main(args):
    """
    Main routine used for command line interface.
    Parameters
    ----------
    args : list of str
        Command line arguments.
    """
    args = parse_args(args)

    reshuffle(args.dataset_root,
              args.timeseries_root,
              args.start,
              args.end,
              args.parameters,
              land_points=args.land_points,
              imgbuffer=args.imgbuffer)



def run():
    main(sys.argv[1:])

if __name__ == '__main__':


    cmd = [r'C:\Temp\tcdr\active_daily', r'C:\Temp\tcdr\ts',
           '1991-08-05', '1991-08-10', '--land_points', 'True']
    main(cmd)

    from interface import C3STs

    ds = C3STs(r'C:\Temp\tcdr\ts')
    ds.read(47.875, 7.875)
    run()


# -*- coding: utf-8 -*-

from c3s_sm.interface import C3SImg
import os
import numpy.testing as nptest
from c3s_sm.grid import C3SLandGrid, C3SCellGrid

# lat=48.125, lon=16.375
def test_C33Ts_tcdr_combined_daily():
    file = os.path.join(os.path.join(os.path.dirname(__file__),
                        'c3s_sm-test-data', 'img', 'TCDR', '060_dailyImages', 'combined', '2014',
                        'C3S-SOILMOISTURE-L3S-SSMV-COMBINED-DAILY-20140101000000-TCDR-v201801.0.0.nc'))


    ds = C3SImg(file, mode='r', parameters='sm', array_1D=False)
    image= ds.read()

    nptest.assert_almost_equal(image.data['sm'][167, 785], 0.34659, 4)
    assert(image.metadata['sm']['long_name'] == 'Volumetric Soil Moisture')
    #assert(file_meta['creator_name'] == 'Earth Observation Data Center (EODC)')



def test_C33Ts_tcdr_active_monthly():

    file = os.path.join(os.path.join(os.path.dirname(__file__),
                        'c3s_sm-test-data', 'img', 'TCDR', '061_monthlyImages', 'active',
                        'C3S-SOILMOISTURE-L3S-SSMS-ACTIVE-MONTHLY-20140101000000-TCDR-v201801.0.0.nc'))


    ds = C3SImg(file, mode='r', parameters='sm', array_1D=False)
    image = ds.read()

    nptest.assert_almost_equal(image.data['sm'][167, 785], 47.69982, 4)
    assert(image.metadata['sm']['_FillValue'] == -9999.)
    assert(image.metadata['sm']['long_name'] == 'Percent of Saturation Soil Moisture')
    #assert(file_meta['creator_name'] == 'Earth Observation Data Center (EODC)')


def test_C33Ts_tcdr_passive_decadal():
    file = os.path.join(os.path.join(os.path.dirname(__file__),
                        'c3s_sm-test-data', 'img', 'TCDR', '062_dekadalImages', 'passive',
                        'C3S-SOILMOISTURE-L3S-SSMV-PASSIVE-DEKADAL-20140101000000-TCDR-v201801.0.0.nc'))


    ds = C3SImg(file, mode='r', parameters='sm', array_1D=False)
    image = ds.read()

    nptest.assert_almost_equal(image['sm'][167, 784], 0.50875, 4)
    assert(image.metadata['sm']['long_name'] == 'Volumetric Soil Moisture')
    #assert(image.metadata['creator_name'] == 'Earth Observation Data Center (EODC)')

################################################################################

def test_C33Ts_icdr_combined_daily():
    file = os.path.join(os.path.join(os.path.dirname(__file__),
                        'c3s_sm-test-data', 'img', 'ICDR', '060_dailyImages', 'combined', '2017',
                        'C3S-SOILMOISTURE-L3S-SSMV-COMBINED-DAILY-20170701000000-ICDR-v201706.0.0.nc'))


    ds = C3SImg(file, mode='r', parameters='sm', array_1D=False)
    image = ds.read()

    nptest.assert_almost_equal(image.data['sm'][167, 785], 0.14548, 4)
    assert(image.metadata['sm']['long_name'] == 'Volumetric Soil Moisture')
    #assert(image['creator_name'] == 'Earth Observation Data Center (EODC)')



def test_C33Ts_icdr_active_monthly():

    file = os.path.join(os.path.join(os.path.dirname(__file__),
                        'c3s_sm-test-data', 'img', 'ICDR', '061_monthlyImages', 'active',
                        'C3S-SOILMOISTURE-L3S-SSMS-ACTIVE-MONTHLY-20170701000000-ICDR-v201706.0.0.nc'))


    ds = C3SImg(file, mode='r', parameters='sm', array_1D=False)
    image = ds.read()

    nptest.assert_almost_equal(image.data['sm'][167, 785], 65.00162, 4)
    assert(image.metadata['sm']['_FillValue'] == -9999.)
    assert(image.metadata['sm']['long_name'] == 'Percent of Saturation Soil Moisture')
    #assert(file_meta['creator_name'] == 'Earth Observation Data Center (EODC)')


def test_C33Ts_icdr_passive_decadal():
    file = os.path.join(os.path.join(os.path.dirname(__file__),
                        'c3s_sm-test-data', 'img', 'ICDR', '062_dekadalImages', 'passive',
                        'C3S-SOILMOISTURE-L3S-SSMV-PASSIVE-DEKADAL-20170701000000-ICDR-v201706.0.0.nc'))


    ds = C3SImg(file, mode='r', parameters='sm', array_1D=False)
    image = ds.read()

    nptest.assert_almost_equal(image.data['sm'][167, 785], 0.21000, 4)
    assert(image.metadata['sm']['long_name'] == 'Volumetric Soil Moisture')
    #assert(image.metadata['creator_name'] == 'Earth Observation Data Center (EODC)')


def test_1Dreading():
    ''' Test 1D reading with and without land grid, and if the results are the same'''
    file = os.path.join(os.path.join(os.path.dirname(__file__),
                        'c3s_sm-test-data', 'img', 'ICDR', '060_dailyImages', 'combined', '2017',
                        'C3S-SOILMOISTURE-L3S-SSMV-COMBINED-DAILY-20170701000000-ICDR-v201706.0.0.nc'))

    ds = C3SImg(file, mode='r', parameters='sm', array_1D=True)
    image = ds.read()

    assert ds.grid.find_nearest_gpi(75.625, 14.625) == (602942, 0)
    ref_sm = image.data['sm'][434462]
    ref_lat = image.lat[434462]
    ref_lon = image.lon[434462]

    assert ref_lat == 14.625
    assert ref_lon == 75.625
    nptest.assert_almost_equal(ref_sm, 0.360762, 5)
    assert(image.metadata['sm']['long_name'] == 'Volumetric Soil Moisture')

    land_grid = C3SLandGrid()

    ds = C3SImg(file, mode='r', parameters='sm', array_1D=True, subgrid=land_grid)
    image = ds.read()

    assert ds.grid.find_nearest_gpi(75.625, 14.625) == (602942, 0)

    sm = image.data['sm'][177048]
    lat = image.lat[177048]
    lon = image.lon[177048]

    assert ref_lat == lat
    assert ref_lon == lon
    nptest.assert_almost_equal(ref_sm, sm, 5)

    assert(image.metadata['sm']['long_name'] == 'Volumetric Soil Moisture')








if __name__ == '__main__':
    test_1Dreading()
    test_C33Ts_tcdr_combined_daily()
    test_C33Ts_tcdr_active_monthly()
    test_C33Ts_tcdr_passive_decadal()

    test_C33Ts_icdr_combined_daily()
    test_C33Ts_icdr_active_monthly()
    test_C33Ts_icdr_passive_decadal()



# -*- coding: utf-8 -*-

import numpy as np

class C3S_SM_TS_Attrs(object):
    '''Default, common metadata for daily and monthly, dekadal products'''
    def __init__(self, product_sensor_type, version='v0000', sub_version='.0.0'):
        # todo: version keyword might be unnecessary?
        '''
        Parameters
        ----------
        product_sensor_type : str
            Sensor type: active, passive, combined
        version : str
            Version name to read attributes for
        sub_version : str
            Sub version to read attributes for
        '''
        self.version, self.version_sub_string = version, sub_version

        self.product_datatype_str = {'active': 'SSMS',
                                     'passive': 'SSMV',
                                     'combined': 'SSMV'}

        self.product_sensor_type = product_sensor_type

        self.atts_product_sensor_type(product_sensor_type)

    def atts_product_sensor_type(self, product_sensor_type='active'):
        if product_sensor_type == "active":
            self.sm_units = "percentage (%)"
            self.sm_uncertainty_units = "percentage (%)"
            self.sm_full_name = 'Percent of Saturation Soil Moisture Uncertainty'
            self.sm_uncertainty_full_name = 'Percent of Saturation Soil Moisture Uncertainty'
        else:
            # todo: some names are wrong? sm_full_name...
            self.sm_units = "m3 m-3"
            self.sm_uncertainty_units = "m3 m-3"
            self.sm_full_name = 'Volumetric Soil Moisture Uncertainty'
            self.sm_uncertainty_full_name = 'Volumetric Soil Moisture Uncertainty'

    def dn_flag(self):
        dn_flag_dict = {0: 'nan',
                        1: "day",
                        2: 'night',
                        3: 'day_night_combination'}
        self.dn_flag_values = np.array(sorted(dn_flag_dict.keys()), dtype=np.byte)
        self.dn_flag_meanings = " ".join([dn_flag_dict[key] for key in self.dn_flag_values])

        return self.dn_flag_values, self.dn_flag_meanings

    def flag(self):
        flag_dict = {0: 'no_data_inconsistency_detected',
                     1: 'snow_coverage_or_temperature_below_zero',
                     2: 'dense_vegetation',
                     3: 'combination_of_flag_values_1_and_2',
                     4: 'others_no_convergence_in_the_model_thus_no_valid_sm_estimates',
                     5: 'combination_of_flag_values_1_and_4',
                     6: 'combination_of_flag_values_2_and_4',
                     7: 'combination_of_flag_values_1_and_2_and_4',
                     8: 'soil_moisture_value_exceeds_physical_boundary',
                     16: 'weight_of_measurement_below_threshold',
                     17: 'combination_of_flag_values_1_and_16',
                     18: 'combination_of_flag_values_2_and_16',
                     19: 'combination_of_flag_values_1_and_2_and_16',
                     20: 'combination_of_flag_values_4_and_16',
                     21: 'combination_of_flag_values_1_and_4_and_16',
                     22: 'combination_of_flag_values_2_and_4_and_16',
                     23: 'combination_of_flag_values_1_and_2_and_4_and_16',
                     127: 'nan'}
        self.flag_values = np.array(sorted(flag_dict.keys()), dtype=np.byte)
        self.flag_meanings = " ".join([flag_dict[key] for key in self.flag_values])

        return self.flag_values, self.flag_meanings

    def freqbandID_flag(self):
        freqbandID_flag_dict = {0: 'NaN',
                                2: 'C53',
                                4: 'C66',
                                8: 'C68',
                                10: 'C53+C68',
                                16: 'C69',
                                18: 'C53+C69',
                                24: 'C68+C69',
                                26: 'C53+C68+C69',
                                32: 'C73',
                                34: 'C53+C73',
                                64: 'X107',
                                66: 'C53+X107',
                                72: 'C68+X107',
                                74: 'C53+C68+X107',
                                80: 'C69+X107',
                                82: 'C53+C69+X107',
                                128: 'K194',
                                130: 'C53+K194'}
        self.freqbandID_flag_values = np.array(sorted(freqbandID_flag_dict.keys()), dtype=np.int)
        self.freqbandID_flag_meanings = " ".join([freqbandID_flag_dict[key] for key in self.freqbandID_flag_values])

        return self.freqbandID_flag_values, self.freqbandID_flag_meanings

    def sensor_flag(self):
        sensor_flag_dict = {0: 'NaN',
                            1: 'SMMR',
                            2: 'SSMI',
                            4: 'TMI',
                            8: 'AMSRE',
                            16: 'WindSat',
                            24: 'AMSRE+WindSat',
                            32: 'AMSR2',
                            128: 'AMIWS',
                            130: 'SSMI+AMIWS',
                            132: 'TMI+AMIWS',
                            136: 'AMSRE+AMIWS',
                            256: 'ASCATA',
                            264: 'AMSRE+ASCATA',
                            272: 'WindSat+ASCATA',
                            280: 'AMSRE+WindSat+ASCATA',
                            288: 'AMSR2+ASCATA',
                            512: 'ASCATB',
                            520: 'AMSRE+ASCATB',
                            528: 'WindSat+ASCATB',
                            536: 'AMSRE+WindSat+ASCATB',
                            544: 'AMSR2+ASCATB',
                            768: 'ASCATA+ASCATB',
                            776: 'AMSRE+ASCATA+ASCATB',
                            784: 'WindSat+ASCATA+ASCATB',
                            792: 'AMSRE+WindSat+ASCATA+ASCATB',
                            800: 'AMSR2+ASCATA+ASCATB'}
        self.sensor_flag_values = np.array(sorted(sensor_flag_dict.keys()), dtype=np.int)
        self.sensor_flag_meanings = " ".join([sensor_flag_dict[key] for key in self.sensor_flag_values])

        return self.sensor_flag_values, self.sensor_flag_meanings

    def mode_flag(self):
        mode_flag_dict = {0: 'nan',
                          1: 'ascending',
                          2: 'descending',
                          3: 'ascending_descending_combination'}
        self.mode_flag_values = np.array(sorted(mode_flag_dict.keys()), dtype=np.int)
        self.mode_flag_meanings = " ".join([mode_flag_dict[key] for key in self.mode_flag_values])

        return self.mode_flag_meanings, self.mode_flag_values


class C3S_daily_tsatt_nc(object):

    def __init__(self, VersionAttrs=C3S_SM_TS_Attrs, product_sub_type='TCDR',
                 product_sensor_type='active', sub_version='.0.0'):

        self.general_attrs = VersionAttrs(product_sensor_type=product_sensor_type,
                                          sub_version=sub_version)

        self.version = self.general_attrs.version
        version_sub_string = self.general_attrs.version_sub_string
        product_sensor_type = self.general_attrs.product_sensor_type

        self.product_temp_res = 'daily'
        self.product_sub_type = product_sub_type
        self.version_sub_string = version_sub_string
        self.general_attrs.atts_product_sensor_type(product_sensor_type)
        self.general_attrs.dn_flag()
        self.general_attrs.flag()
        self.general_attrs.freqbandID_flag()
        self.general_attrs.mode_flag()
        self.general_attrs.sensor_flag()

        self.ts_attributes = {
            'dnflag': {'full_name': 'Day / Night Flag',
                       'units': '1',
                       'flag_values': self.general_attrs.dn_flag_values,
                       'flag_meanings': self.general_attrs.dn_flag_meanings},
            'flag': {'full_name': 'Flag',
                     'units': '1',
                     'flag_values': self.general_attrs.flag_values,
                     'flag_meanings': self.general_attrs.flag_meanings},
            'freqbandID': {'full_name': 'Frequency Band Identification',
                           'units': '1',
                           'flag_values': self.general_attrs.freqbandID_flag_values,
                           'flag_meanings': self.general_attrs.freqbandID_flag_meanings},
            'mode': {'full_name': 'Satellite  Mode',
                     'units': '1',
                     'flag_values': self.general_attrs.mode_flag_values,
                     'flag_meanings': self.general_attrs.mode_flag_meanings},
            'sensor': {'full_name': 'Sensor',
                       'units': '1',
                       'flag_values': self.general_attrs.sensor_flag_values,
                       'flag_meanings': self.general_attrs.sensor_flag_meanings},
            'sm': {'full_name': self.general_attrs.sm_full_name,
                   'units': self.general_attrs.sm_units},
            'sm_uncertainty': {'full_name': self.general_attrs.sm_uncertainty_full_name,
                               'units': self.general_attrs.sm_uncertainty_units},
            't0': {'full_name': 'Observation Timestamp',
                   'units': 'days since 1970-01-01 00:00:00 UTC'}}

        product_name = "-".join(['C3S', 'SOILMOISTURE', 'L3S',
                                 self.general_attrs.product_datatype_str[product_sensor_type].upper(),
                                 product_sensor_type.upper(), self.product_temp_res.upper(),
                                 self.product_sub_type.upper(),
                                 self.version + self.version_sub_string])

        self.global_attr = {'product': product_name,
                            'resolution': '0.25 degree',
                            'temporalspacing': self.product_temp_res}


class C3S_dekmon_tsatt_nc(object):
    '''	Attributes for c3s dekadal and monthly for active, passive and combined
    tcdr and icdr timeseries files.'''

    def __init__(self, VersionAttrs, product_temp_res='monthly',
                 product_sub_type='TCDR', product_sensor_type='active', sub_version='.0.0'):

        self.general_attrs = VersionAttrs(product_sensor_type=product_sensor_type,
                                          sub_version=sub_version)


        self.version = self.general_attrs.version
        version_sub_string = self.general_attrs.version_sub_string
        product_sensor_type = self.general_attrs.product_sensor_type

        self.product_temp_res = product_temp_res
        self.product_sub_type = product_sub_type
        self.version_sub_string = version_sub_string
        self.general_attrs.atts_product_sensor_type(product_sensor_type)
        self.general_attrs.dn_flag()
        self.general_attrs.freqbandID_flag()

        self.general_attrs.sensor_flag()



        self.ts_attributes = {
            'freqbandID': {'full_name': 'Frequency Band Identification',
                           'units': '1',
                           'flag_values': self.general_attrs.freqbandID_flag_values,
                           'flag_meanings': self.general_attrs.freqbandID_flag_meanings},
            'sensor': {'full_name': 'Sensor',
                       'units': '1',
                       'flag_values': self.general_attrs.sensor_flag_values,
                       'flag_meanings': self.general_attrs.sensor_flag_meanings},
            'nobs': {'full_name': 'Number of valid observation'},
            'sm': {'full_name': self.general_attrs.sm_full_name,
                   'units': self.general_attrs.sm_units}}

        product_name = "-".join(['C3S', 'SOILMOISTURE', 'L3S',
                                 self.general_attrs.product_datatype_str[product_sensor_type].upper(),
                                 product_sensor_type.upper(), self.product_temp_res.upper(),
                                 self.product_sub_type.upper(),
                                 self.version + self.version_sub_string])

        self.global_attr = {'product': product_name,
                            'resolution': '0.25 degree',
                            'temporalspacing': self.product_temp_res}


class C3S_SM_TS_Attrs_v201706(C3S_SM_TS_Attrs):
    # Example for a version specific attribute class, last part defines version
    def __init__(self, product_sensor_type, sub_version='.0.0'):

        version = type(self).__name__.split('_')[-1]
        super(C3S_SM_TS_Attrs_v201706, self).__init__(product_sensor_type, version,
                                                      sub_version)


class C3S_SM_TS_Attrs_v201801(C3S_SM_TS_Attrs):
    # Example for a version specific attribute class, last part defines version
    def __init__(self, product_sensor_type, sub_version='.0.0'):

        version = type(self).__name__.split('_')[-1]
        super(C3S_SM_TS_Attrs_v201801, self).__init__(product_sensor_type, version,
                                                      sub_version)

    # TODO: Version specific values can be changed here by changing the functions
    def freqbandID_flag(self):
        freqbandID_flag_dict = {0: 'NaN',
                                2: 'C53',
                                4: 'C66',
                                8: 'C68',
                                10: 'C53+C68',
                                16: 'C69',
                                18: 'C53+C69',
                                24: 'C68+C69',
                                26: 'C53+C68+C69',
                                32: 'C73',
                                34: 'C53+C73',
                                64: 'X107',
                                66: 'C53+X107',
                                72: 'C68+X107',
                                74: 'C53+C68+X107',
                                80: 'C69+X107',
                                82: 'C53+C69+X107',
                                128: 'K194',
                                130: 'C53+K194'}
        self.freqbandID_flag_values = np.array(sorted(freqbandID_flag_dict.keys()), dtype=np.int)
        self.freqbandID_flag_meanings = " ".join([freqbandID_flag_dict[key] for key in self.freqbandID_flag_values])

        return self.freqbandID_flag_values, self.freqbandID_flag_meanings



if __name__ == '__main__':
    o = C3S_SM_TS_Attrs_v201801('active')

    dob = C3S_dekmon_tsatt_nc(C3S_SM_TS_Attrs_v201801, product_sub_type='TCDR',
                 product_sensor_type='active', product_temp_res='monthly', sub_version='.9.9')

    attr = C3S_SM_TS_Attrs_v201801('active')




# -*- coding: utf-8 -*-

import os
import glob
import tempfile
import numpy as np
import numpy.testing as nptest

from c3s_sm.reshuffle import main, parse_filename
from c3s_sm.interface import C3STs

def test_parse_filename():
    inpath = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                          "c3s_sm-test-data", "img2ts", "ICDR", "combined")

    file_args, file_vars = parse_filename(inpath)


    assert file_args['product'] == 'C3S'
    assert file_args['data_type'] == 'SSMV'
    assert file_args['sensor_type'] == 'COMBINED'
    assert file_args['temp_res'] == 'MONTHLY'
    assert file_args['sub_prod'] == 'ICDR'
    assert file_args['version'] == 'v201706'
    assert file_args['sub_version'] == '0.0'

    assert file_vars == [u'lat', u'lon', u'time', u'nobs', u'sensor', u'freqbandID', u'sm']




def test_reshuffle_TCDR_daily_multiple_params():
    inpath = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                          "c3s_sm-test-data", "img2ts", "TCDR", "active")
    startdate = "1991-08-05"
    enddate = "1991-08-08"
    #parameters = ["sm", "sm_uncertainty", "dnflag", "flag", "freqbandID", "mode", "sensor", "t0"]
    parameters = ['--parameters', 'sm', 'sm_uncertainty']
    land_points = 'True'

    ts_path = tempfile.mkdtemp()
    args = [inpath, ts_path, startdate, enddate]  + \
           parameters + ['--land_points', land_points]
    main(args)

    assert len(glob.glob(os.path.join(ts_path, "*.nc"))) == 1002

    ds = C3STs(ts_path, remove_nans=True, parameters=['sm', 'sm_uncertainty'], ioclass_kws={'read_bulk': True, 'read_dates': False})
    ts = ds.read(75.625, 14.625)
    ts_sm_values_should = np.array([66.0677, np.nan, 80.7060, 70.5648], dtype=np.float32)
    nptest.assert_allclose(ts['sm'].values, ts_sm_values_should, rtol=1e-5)

    ts_uncert_values_should = np.array([np.nan, np.nan, np.nan, np.nan],
                                       dtype=np.float32)
    nptest.assert_allclose(ts['sm_uncertainty'].values, ts_uncert_values_should,rtol=1e-5)

    nptest.assert_almost_equal(ts['sm'].values, ds.read(602942)['sm'].values)

    #nptest.assert_allclose(ts['dnflag'].values, [1,0,2,1],rtol=1e-5)
    #nptest.assert_allclose(ts['flag'].values, [0,127,0,0],rtol=1e-5)
    #nptest.assert_allclose(ts['freqbandID'].values, [2,0,2,2],rtol=1e-5)
    #nptest.assert_allclose(ts['mode'].values, [2,0,1,2],rtol=1e-5)
    #nptest.assert_allclose(ts['sensor'].values, [128,0, 128,128],rtol=1e-5)
    #nptest.assert_almost_equal(ts['t0'].values, [7886.22702546, -3440586.5, 7887.7328588,
     #                                     7889.22702546])

    ds.close()

def test_reshuffle_ICDR_monthly_single_param():
    inpath = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                          "c3s_sm-test-data", "img2ts", "ICDR", "combined")
    startdate = "2018-05-01"
    enddate = "2018-08-01"
    parameters = ['--parameters', "sm"] #, "sensor"]

    land_points = 'False'

    ts_path = tempfile.mkdtemp()
    args = [inpath, ts_path, startdate, enddate] \
           + parameters + ['--land_points', land_points]
    main(args)


    assert len(glob.glob(os.path.join(ts_path, "*.nc"))) == 2593

    ds = C3STs(ts_path, remove_nans=True, parameters=['sm'], ioclass_kws={'read_bulk': True, 'read_dates': False})
    ts = ds.read(-159.625, 65.875)
    ts_sm_values_should = np.array([0.23628984, 0.33424062, np.nan, 0.26261818], dtype=np.float32)

    nptest.assert_allclose(ts['sm'].values, ts_sm_values_should, rtol=1e-5)

    #ts_sensor_values_should = np.array([768, 768, 768, 768 ], dtype=np.float32)
    #nptest.assert_allclose(ts['sensor'].values, ts_sensor_values_should,rtol=1e-5)

    ds.close()



if __name__ == '__main__':
    test_parse_filename()
    test_reshuffle_ICDR_monthly_single_param()
    test_reshuffle_TCDR_daily_multiple_params()




# -*- coding: utf-8 -*-

from c3s_sm.metadata import C3S_daily_tsatt_nc, C3S_SM_TS_Attrs, C3S_dekmon_tsatt_nc


def test_daily_metadata_default_active():

    default_attr = C3S_SM_TS_Attrs('active')

    assert(default_attr.version == 'v0000')
    assert(default_attr.product_sensor_type == 'active')
    assert(default_attr.version_sub_string == '.0.0')
    assert(default_attr.sm_full_name == 'Percent of Saturation Soil Moisture Uncertainty')
    assert(default_attr.sm_units == "percentage (%)")

    default_attr.flag()
    assert(default_attr.flag_values[0] == 0)
    assert(default_attr.flag_meanings.split(' ')[0] == 'no_data_inconsistency_detected')
    assert(default_attr.flag_values[10] == 17)
    assert(default_attr.flag_meanings.split(' ')[10] == 'combination_of_flag_values_1_and_16')

    default_attr.freqbandID_flag()
    assert (default_attr.freqbandID_flag_values[0] == 0)
    assert (default_attr.freqbandID_flag_meanings.split(' ')[0] == 'NaN')

    assert (default_attr.freqbandID_flag_values[10] == 34)
    assert (default_attr.freqbandID_flag_meanings.split(' ')[10] == 'C53+C73')

    default_attr.sensor_flag()
    assert (default_attr.sensor_flag_values[0] == 0)
    assert (default_attr.sensor_flag_meanings.split(' ')[0] == 'NaN')

    assert (default_attr.sensor_flag_values[10] == 132)
    assert (default_attr.sensor_flag_meanings.split(' ')[10] == 'TMI+AMIWS')

    default_attr.mode_flag()
    assert (default_attr.mode_flag_values[0] == 0)
    assert (default_attr.mode_flag_meanings.split(' ')[0] == 'nan')

    assert (default_attr.mode_flag_values[3] == 3)
    assert (default_attr.mode_flag_meanings.split(' ')[3] == 'ascending_descending_combination')


def test_daily_metadata_default_passive_and_combined():
    for sensor in ['passive', 'combined']:
        default_attr = C3S_SM_TS_Attrs(sensor)

        assert (default_attr.version == 'v0000')
        assert (default_attr.product_sensor_type == sensor)
        assert (default_attr.version_sub_string == '.0.0')
        assert (default_attr.sm_full_name == 'Volumetric Soil Moisture Uncertainty')
        assert (default_attr.sm_units == "m3 m-3")

        default_attr.flag()
        assert (default_attr.flag_values[0] == 0)
        assert (default_attr.flag_meanings.split(' ')[0] == 'no_data_inconsistency_detected')
        assert (default_attr.flag_values[10] == 17)
        assert (default_attr.flag_meanings.split(' ')[10] == 'combination_of_flag_values_1_and_16')

        default_attr.freqbandID_flag()
        assert (default_attr.freqbandID_flag_values[0] == 0)
        assert (default_attr.freqbandID_flag_meanings.split(' ')[0] == 'NaN')

        assert (default_attr.freqbandID_flag_values[10] == 34)
        assert (default_attr.freqbandID_flag_meanings.split(' ')[10] == 'C53+C73')

        default_attr.sensor_flag()
        assert (default_attr.sensor_flag_values[0] == 0)
        assert (default_attr.sensor_flag_meanings.split(' ')[0] == 'NaN')

        assert (default_attr.sensor_flag_values[10] == 132)
        assert (default_attr.sensor_flag_meanings.split(' ')[10]== 'TMI+AMIWS')

        default_attr.mode_flag()
        assert (default_attr.mode_flag_values[0] == 0)
        assert (default_attr.mode_flag_meanings.split(' ')[0] == 'nan')

        assert (default_attr.flag_values[3] == 3)
        assert (default_attr.mode_flag_meanings.split(' ')[3] == 'ascending_descending_combination')

def test_C3s_daily_tsatt_nc():
    subtype = 'TCDR'
    sensor = 'active'
    dob = C3S_daily_tsatt_nc(C3S_SM_TS_Attrs, product_sub_type=subtype,
                 product_sensor_type=sensor, sub_version='.9.9')

    glob = dob.global_attr

    assert glob == {'product': 'C3S-SOILMOISTURE-L3S-SSMS-%s-%s-%s-v0000.9.9'
                               %(sensor.upper(), 'DAILY', subtype),
                    'resolution': '0.25 degree',
                    'temporalspacing': 'daily'}
    assert dob.ts_attributes['flag']['flag_values'].size == 18

    sm_should = {'units': 'percentage (%)', 'full_name': 'Percent of Saturation Soil Moisture Uncertainty'}
    assert dob.ts_attributes['sm'] == sm_should

    assert dob.ts_attributes['mode']['flag_values'].size == 4

    t0_should = {'units': 'days since 1970-01-01 00:00:00 UTC', 'full_name': 'Observation Timestamp'}
    assert dob.ts_attributes['t0'] == t0_should

def test_C3s_dekmon_tsatt_nc():
    subtype = 'TCDR'
    sensor = 'passive'
    dob = C3S_dekmon_tsatt_nc(C3S_SM_TS_Attrs, product_sub_type=subtype,
                             product_sensor_type=sensor, sub_version='.9.9')

    glob = dob.global_attr

    assert glob == {'product': 'C3S-SOILMOISTURE-L3S-SSMV-%s-%s-%s-v0000.9.9'
                               % (sensor.upper(), 'MONTHLY', subtype),
                    'resolution': '0.25 degree',
                    'temporalspacing': 'monthly'}
    assert dob.ts_attributes['freqbandID']['flag_values'].size == 19

    sm_should = {'units': 'm3 m-3', 'full_name': 'Volumetric Soil Moisture Uncertainty'}
    assert dob.ts_attributes['sm'] == sm_should

    assert dob.ts_attributes['nobs'] == {'full_name': 'Number of valid observation'}

    t0_should = {'units': 'days since 1970-01-01 00:00:00 UTC', 'full_name': 'Observation Timestamp'}
    assert dob.ts_attributes['sensor']['flag_values'].size == 27


if __name__ == '__main__':
    test_C3s_dekmon_tsatt_nc()
    test_C3s_daily_tsatt_nc()
    test_daily_metadata_default_passive_and_combined()
    test_daily_metadata_default_active()


# -*- coding: utf-8 -*-

from c3s_sm.grid import C3SCellGrid
from c3s_sm.grid import C3SLandGrid
import numpy as np


def test_C3SCellGrid():
    grid = C3SCellGrid()
    gp, dist = grid.find_nearest_gpi(75.625, 14.625)
    assert gp == 602942
    lon, lat = grid.gpi2lonlat(602942)
    assert lon == 75.625
    assert lat == 14.625
    assert np.where(grid.get_grid_points()[0] == 602942)[0][0] == 434462 # index
    assert grid.get_grid_points()[1][434462] == lon
    assert grid.get_grid_points()[2][434462] == lat
    assert grid.gpi2cell(602942) == 1856
    assert grid.gpis.size == 1036800
    assert grid.gpis[0] == 1035360
    assert np.unique(grid.get_grid_points()[3]).size == 2592


def test_landgrid():
    grid = C3SLandGrid()
    gp, dist = grid.find_nearest_gpi(75.625, 14.625)
    assert gp == 602942
    lon, lat = grid.gpi2lonlat(602942)
    assert lon == 75.625
    assert lat == 14.625
    assert np.where(grid.get_grid_points()[0] == 602942)[0][0] == 177048  # index
    assert grid.get_grid_points()[1][177048] == lon
    assert grid.get_grid_points()[2][177048] == lat
    assert grid.gpi2cell(602942) == 1856
    assert grid.gpis.size == 1036800
    assert grid.activegpis.size == 244243
    assert grid.gpis[0] == 1035360
    assert grid.activegpis[0] == 999942
    assert np.unique(grid.get_grid_points()[3]).size == 1001




# The MIT License (MIT)
#
# Copyright (c) 2018, TU Wien
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

'''
Readers for the C3S soil moisture proudct daily, dekadal (10-daily) and monthly images as well
as for timeseries generated using this module
'''

import pandas as pd
import os
import netCDF4 as nc
import numpy as np
from datetime import timedelta
from dateutil.relativedelta import relativedelta

from pygeobase.object_base import Image
from pygeobase.io_base import ImageBase
from pygeobase.io_base import MultiTemporalImageBase
from pygeogrids.netcdf import load_grid

from netCDF4 import Dataset
from pynetcf.time_series import GriddedNcOrthoMultiTs
from datetime import datetime
from parse import parse
from c3s_sm.grid import C3SCellGrid, C3SLandGrid


def c3s_filename_template():
    # this function can be used in case the filename changes at some point.
    return '{product}-SOILMOISTURE-L3S-{data_type}-{sensor_type}-{temp_res}-' \
           '{datetime}000000-{sub_prod}-{version}.{sub_version}.nc'


class C3STs(GriddedNcOrthoMultiTs):
    """
    Module for reading C3S time series in netcdf format.
    """

    def __init__(self, ts_path, grid_path=None, remove_nans=False, **kwargs):

        '''
        Class for reading C3S SM time series after reshuffling.

        Parameters
        ----------
        ts_path : str
            Directory where the netcdf time series files are stored
        grid_path : str, optional (default: None)
            Path to grid file, that is used to organize the location of time
            series to read. If None is passed, grid.nc is searched for in the
            ts_path.
        remove_nans : bool, optional (default: False)
            Replace -9999 with np.nan in time series

        Optional keyword arguments that are passed to the Gridded Base:
        ------------------------------------------------------------------------
            parameters : list, optional (default: None)
                Specific variable names to read, if None are selected, all are read.
            offsets : dict, optional (default:None)
                Offsets (values) that are added to the parameters (keys)
            scale_factors : dict, optional (default:None)
                Offset (value) that the parameters (key) is multiplied with
            ioclass_kws: dict
                Optional keyword arguments to pass to OrthoMultiTs class:
                ----------------------------------------------------------------
                    read_bulk : boolean, optional (default:False)
                        if set to True the data of all locations is read into memory,
                        and subsequent calls to read_ts read from the cache and not from disk
                        this makes reading complete files faster#
                    read_dates : boolean, optional (default:False)
                        if false dates will not be read automatically but only on specific
                        request useable for bulk reading because currently the netCDF
                        num2date routine is very slow for big datasets
        '''
        self.remove_nans = remove_nans

        if grid_path is None:
            grid_path = os.path.join(ts_path, "grid.nc")

        grid = load_grid(grid_path)

        super(C3STs, self).__init__(ts_path, grid=grid, **kwargs)


    def _read_gp(self, gpi, **kwargs):
        """Read a single point from passed gpi or from passed lon, lat """
        # override the _read_gp function from parent class, to add dropna functionality

        ts = super(C3STs, self)._read_gp(gpi, **kwargs)
        if ts is None:
            return None

        if self.remove_nans:
            ts = ts.replace(-9999.0000, np.nan)

        ts.index = ts.index.tz_localize('UTC')

        return ts


    def read_cell(self, cell, var='sm'):
        """
        Read all time series for a single variable in the selected cell.

        Parameters
        -------
        cell: int
            Cell number as in the c3s grid
        var : str, optional (default: 'sm')
            Name of the variable to read.
        """

        file_path = os.path.join(self.path, '{}.nc'.format("%04d" % (cell,)))
        with nc.Dataset(file_path) as ncfile:
            loc_id = ncfile.variables['location_id'][:]
            time = ncfile.variables['time'][:]
            unit_time = ncfile.variables['time'].units
            delta = lambda t: timedelta(t)
            vfunc = np.vectorize(delta)
            since = pd.Timestamp(unit_time.split('since ')[1])
            time = since + vfunc(time)

            variable = ncfile.variables[var][:]
            variable = np.transpose(variable)
            data = pd.DataFrame(variable, columns=loc_id, index=time)
            if self.remove_nans:
                data = data.replace(-9999.0000, np.nan)
            return data


class C3SImg(ImageBase):
    """
    Class to read a single C3S image (for one time stamp)
    """
    def __init__(self, filename, parameters='sm', mode='r', subgrid=None,
                 array_1D=False):
        # todo: get rid of mode?
        '''
        Parameters
        ----------
        filename : str
            Path to the file to read
        parameters : str or Iterable, optional (default: 'sm')
            Names of parameters in the file to read.
            If None are passed, all are read.
        mode : str, optional (default: 'r')
            Netcdf file mode, choosing something different to r may delete data.
        array_1D : bool, optional (default: False)
            Read image as one dimensional array, instead of a 2D array
            Use this when using a subgrid.
        '''

        super(C3SImg, self).__init__(filename, mode=mode)

        if not isinstance(parameters, list):
            parameters = [parameters]

        self.parameters = parameters
        self.grid = C3SCellGrid() if not subgrid else subgrid
        self.array_1D = array_1D

    def read(self, timestamp=None):
        """
        Reads a single C3S image.

        Parameters
        -------
        timestamp: datetime, optional (default: None)
            Timestamp for file to read. Pass None if file contains only 1 timestamp

        Returns
        -------
        image : Image
            Image object from netcdf content
        """

        ds = Dataset(self.filename, mode='r')

        param_img = {}
        img_meta = {'global': {}}

        if self.parameters[0] is None:
            parameters = ds.variables.keys()
        else:
            parameters = self.parameters

        for param in parameters:
            if param in ['lat', 'lon', 'time']: continue
            param_metadata = {}

            variable = ds.variables[param]

            for attr in variable.ncattrs():
                param_metadata.update({str(attr): getattr(variable, attr)})

            param_data = np.flipud(variable[0][:].filled()).flatten()

            param_img[str(param)] = param_data[self.grid.activegpis]
            img_meta[param] = param_metadata

        # add global attributes
        for attr in ds.ncattrs():
            img_meta['global'][attr] = ds.getncattr(attr)

        ds.close()

        if self.array_1D:
            return Image(self.grid.activearrlon, self.grid.activearrlat,
                         param_img, img_meta, timestamp)
        else:
            yres, xres = self.grid.shape
            for key in param_img:
                param_img[key] = param_img[key].reshape(xres, yres)

            return Image(self.grid.activearrlon.reshape(xres, yres),
                         self.grid.activearrlat.reshape(xres, yres),
                         param_img,
                         img_meta,
                         timestamp)


    def write(self, *args, **kwargs):
        pass

    def close(self, *args, **kwargs):
        pass

    def flush(self, *args, **kwargs):
        pass



class C3S_Nc_Img_Stack(MultiTemporalImageBase):
    '''Class for reading multiple images and iterate over them. '''
    def __init__(self, data_path, parameters='sm', subgrid=None, array_1D=False):
        '''
        Parameters
        ----------
        data_path : str
            Path to directory where C3S images are stored
        parameters : list or str,  optional (default: 'sm')
            Variables to read from the image files.
        subgrid : grid, optional (default: None)
            Subset of the image to read
        array_1D : bool, optional (default: False)
            Flatten the read image to a 1D array instead of a 2D array
        '''

        self.data_path = data_path
        ioclass_kwargs = {'parameters': parameters,
                          'subgrid' : subgrid,
                          'array_1D': array_1D}

        template = c3s_filename_template()
        self.fname_args = self._parse_filename(template)
        filename_templ = template.format(**self.fname_args)

        # todo: is this fixed? Daily data is organised differently than M and 10D?
        if self.fname_args['temp_res'] == 'DAILY':
            subpath_templ = ['%Y']
        else:
            subpath_templ = None

        super(C3S_Nc_Img_Stack, self).__init__(path=data_path, ioclass=C3SImg,
                                               fname_templ=filename_templ ,
                                               datetime_format="%Y%m%d",
                                               subpath_templ=subpath_templ,
                                               exact_templ=True,
                                               ioclass_kws=ioclass_kwargs)

    def _parse_filename(self, template):
        '''
        Search a file in the passed directory and use the filename template to
        to read settings.

        Parameters
        -------
        template : str
            Template for all files in the passed directory.

        Returns
        -------
        parse_result : parse.Result
            Parsed content of filename string from filename template.
        '''

        for curr, subdirs, files in os.walk(self.data_path):
            for f in files:
                file_args = parse(template, f)
                if file_args is None:
                    continue
                else:
                    file_args = file_args.named
                    file_args['datetime'] = '{datetime}'
                    return file_args

        raise IOError('No file name in passed directory fits to template')



    def tstamps_for_daterange(self, start_date, end_date):
        '''
        Return dates in the passed period, with respect to the temp resolution
        of the images in the path.

        Parameters
        ----------
        start_date: datetime
            start of date range
        end_date: datetime
            end of date range

        Returns
        -------
        timestamps : list
            list of datetime objects of each available image between
            start_date and end_date
        '''

        if self.fname_args['temp_res'] == 'MONTHLY':
            next = lambda date : date + relativedelta(months=1)
        elif self.fname_args['temp_res'] == 'DAILY':
            next = lambda date : date + relativedelta(days=1)
        elif self.fname_args['temp_res'] == 'DEKADAL':
            next = lambda date : date + relativedelta(days=10)
        else:
            raise NotImplementedError

        timestamps = [start_date]
        while next(timestamps[-1]) <= end_date:
            timestamps.append(next(timestamps[-1]))

        return timestamps





